package com.jh.boot.template;

import java.io.StringWriter;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

public class HibernateLoaderTemplateService implements TemplateService {

	private static final Logger LOG = LoggerFactory.getLogger(HibernateLoaderTemplateService.class);
	
	private TemplateLoader loader;
	
	private TemplateEngine engine;
	
	private TemplateValuesConfig valuesConfig;
	
	@Override
	@Transactional
	public String evaluate(String templateCode, Map<String, Object> data, Locale locale) {
		Template t = loader.loadTemplate(templateCode, locale);
		if (t == null) {
			LOG.warn("Cannot find template {}", templateCode);
			return null;
		}
		
		Map<String, Object> values = new HashMap<String, Object>();
		if (valuesConfig != null) {
			values.putAll(valuesConfig.getValues());
		}
		values.putAll(data);
		
		String content = t.getTemplate();
		StringWriter sw = new StringWriter();
		engine.evaluate(sw, content, values);
		content = sw.toString();
		
		if (t.getLayoutCode() != null) {
			t = loader.loadTemplate(t.getLayoutCode(), locale);
			if (t != null) {
				values.put("__layout_content__", content);
			}
			sw = new StringWriter();
			engine.evaluate(sw, t.getTemplate(), values);
			content = sw.toString();
		}
		
		return content;
	}
	
	@Autowired
	public void setLoader(TemplateLoader loader) {
		this.loader = loader;
	}

	@Autowired
	public void setEngine(TemplateEngine engine) {
		this.engine = engine;
	}

	@Autowired
	public void setValuesConfig(TemplateValuesConfig valuesConfig) {
		this.valuesConfig = valuesConfig;
	}

	
	
}
